﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using Microsoft.AspNetCore.Mvc;
using groupH_M32COM.Api.Models;
using Microsoft.Extensions.Configuration;
using Microsoft.IdentityModel.Tokens;
using System.IdentityModel.Tokens.Jwt;
using System.Text;
using System.Security.Claims;
using Microsoft.AspNetCore.Authorization;

namespace groupH_M32COM.Api.Controllers
{
    [Authorize]
    [Route("api/[controller]")]
    [ApiController]
    public class UsersController : ControllerBase
    {
        private readonly UserContext _context;

        public UsersController(UserContext context)
        {
            _context = context;
        }

        // POST api/authenticate
        [AllowAnonymous]
        [HttpPost("authenticate")]
        public ActionResult<User> Authenticate([FromBody] LoginModel login)
        {
            User user = _context.Users.FirstOrDefault(u => u.Username == login.Username);

            // Validate password (it's case sensitive)
            if (user == null || user.Password != login.Password)
            {
                return new BadRequestResult();
            }

            // Remove password
            user.Password = null;

            var tokenHandler = new JwtSecurityTokenHandler();
            var key = Encoding.ASCII.GetBytes(Settings.SECRET);
            var tokenDescriptor = new SecurityTokenDescriptor()
            {
                Subject = new System.Security.Claims.ClaimsIdentity(new Claim[]
                {
                    new Claim(ClaimTypes.Name, user.Username),
                    new Claim(ClaimTypes.Role, user.Role)
                }),
                Expires = DateTime.UtcNow.AddDays(10),
                SigningCredentials = new SigningCredentials(new SymmetricSecurityKey(key), SecurityAlgorithms.HmacSha256Signature)
            };

            var token = tokenHandler.CreateToken(tokenDescriptor);
            user.Token = tokenHandler.WriteToken(token);

            return user;
        }

        // GET api/users/AdminEndpoint
        [Authorize(Roles = Role.Admin)]
        [HttpGet("AdminEndpoint")]
        public ActionResult<string> AdminEndpoint()
        {
            return "Hello, Admin! You are authorized to access this endpoint.";
        }

        // GET api/users/UserEndpoint
        [Authorize(Roles = Role.User)]
        [HttpGet("UserEndpoint")]
        public ActionResult<string> UserEndpoint()
        {
            return "Hello, User! You are authorized to access this endpoint.";
        }
    }
}
